@@ -1,8 +1,10 @@
 ;;; nnsoup.el --- SOUP access for Gnus
-;; Copyright (C) 1995,96,97,98,99 Free Software Foundation, Inc.
+
+;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003
+;;	Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
-;; 	Masanobu UMEDA <umerin@flab.flab.fujitsu.junet>
+;;	Masanobu UMEDA <umerin@flab.flab.fujitsu.junet>
 ;; Keywords: news, mail
 
 ;; This file is part of GNU Emacs.
@@ -44,7 +46,7 @@
 	  ("/tmp/"))
   "*Where nnsoup will store temporary files.")
 
-(defvoo nnsoup-replies-directory (concat nnsoup-directory "replies/")
+(defvoo nnsoup-replies-directory (expand-file-name "replies/" nnsoup-directory)
   "*Directory where outgoing packets will be composed.")
 
 (defvoo nnsoup-replies-format-type ?u  ;; u is USENET news format.
@@ -53,7 +55,7 @@
 (defvoo nnsoup-replies-index-type ?n
   "*Index type of the replies packages.")
 
-(defvoo nnsoup-active-file (concat nnsoup-directory "active")
+(defvoo nnsoup-active-file (expand-file-name "active" nnsoup-directory)
   "Active file.")
 
 (defvoo nnsoup-packer "tar cf - %s | gzip > $HOME/Soupin%d.tgz"
@@ -112,7 +114,7 @@ backend for the messages.")
 	;; articles in SEQUENCE come from.
 	(while (and areas sequence)
 	  ;; Peel off areas that are below sequence.
-	  (while (and areas (< (cdaar areas) (car sequence)))
+	  (while (and areas (< (cdar (car areas)) (car sequence)))
 	    (setq areas (cdr areas)))
 	  (when areas
 	    ;; This is a useful area.
@@ -128,7 +130,7 @@ backend for the messages.")
 	      (setq use-nov nil))
 	    ;; We assign the portion of `sequence' that is relevant to
 	    ;; this MSG packet to this packet.
-	    (while (and sequence (<= (car sequence) (cdaar areas)))
+	    (while (and sequence (<= (car sequence) (cdar (car areas))))
 	      (push (car sequence) this-area-seq)
 	      (setq sequence (cdr sequence)))
 	    (setcar useful-areas (cons (nreverse this-area-seq)
@@ -156,7 +158,7 @@ backend for the messages.")
 		  (when index-buffer
 		    (insert-buffer-substring index-buffer)
 		    (goto-char b)
-		    ;; We have to remove the index number entires and
+		    ;; We have to remove the index number entries and
 		    ;; insert article numbers instead.
 		    (while (looking-at "[0-9]+")
 		      (replace-match (int-to-string number) t t)
@@ -247,7 +249,7 @@ backend for the messages.")
   ;; Try to guess the type based on the first article in the group.
   (when (not article)
     (setq article
-	  (cdaar (cddr (assoc group nnsoup-group-alist)))))
+	  (cdar (car (cddr (assoc group nnsoup-group-alist))))))
   (if (not article)
       'unknown
     (let ((kind (gnus-soup-encoding-kind
@@ -255,7 +257,7 @@ backend for the messages.")
 		  (nth 1 (nnsoup-article-to-area
 			  article nnsoup-current-group))))))
       (cond ((= kind ?m) 'mail)
-	    ((= kind ?n) 'news) 
+	    ((= kind ?n) 'news)
 	    (t 'unknown)))))
 
 (deffoo nnsoup-close-group (group &optional server)
@@ -335,7 +337,7 @@ backend for the messages.")
 		  (delete-file (nnsoup-file prefix t)))
 		t)
 	  (setcdr (cdr total-infolist) (delq info (cddr total-infolist)))
-	  (setq articles (gnus-sorted-complement articles range-list))))
+	  (setq articles (gnus-sorted-difference articles range-list))))
       (when (not mod-time)
 	(setcdr (cdr total-infolist) (delq info (cddr total-infolist)))))
     (if (cddr total-infolist)
@@ -369,7 +371,7 @@ backend for the messages.")
 	  (setq min (caaar e))
 	  (while (cdr e)
 	    (setq e (cdr e)))
-	  (setq max (cdaar e))
+	  (setq max (cdar (car e)))
 	  (setcdr entry (cons (cons min max) (cdr entry)))))
       (setq nnsoup-group-alist-touched t))
     nnsoup-group-alist))
@@ -397,7 +399,7 @@ backend for the messages.")
     prefix))
 
 (defun nnsoup-file-name (dir file)
-  "Return the full path of FILE (in any case) in DIR."
+  "Return the full name of FILE (in any case) in DIR."
   (let* ((case-fold-search t)
 	 (files (directory-files dir t))
 	 (regexp (concat (regexp-quote file) "$")))
@@ -422,12 +424,15 @@ backend for the messages.")
 	    (setq cur-prefix (nnsoup-next-prefix))
 	    (nnheader-message 5 "Incorporating file %s..." cur-prefix)
 	    (when (file-exists-p
-		   (setq file (concat nnsoup-tmp-directory
-				      (gnus-soup-area-prefix area) ".IDX")))
+		   (setq file
+			 (expand-file-name
+			  (concat (gnus-soup-area-prefix area) ".IDX")
+			  nnsoup-tmp-directory)))
 	      (rename-file file (nnsoup-file cur-prefix)))
 	    (when (file-exists-p
-		   (setq file (concat nnsoup-tmp-directory
-				      (gnus-soup-area-prefix area) ".MSG")))
+		   (setq file (expand-file-name
+			       (concat (gnus-soup-area-prefix area) ".MSG")
+			       nnsoup-tmp-directory)))
 	      (rename-file file (nnsoup-file cur-prefix t))
 	      (gnus-soup-set-area-prefix area cur-prefix)
 	      ;; Find the number of new articles in this area.
@@ -531,17 +536,19 @@ backend for the messages.")
   (let* ((file (concat prefix (if message ".MSG" ".IDX")))
 	 (buffer-name (concat " *nnsoup " file "*")))
     (or (get-buffer buffer-name)	; File already loaded.
-	(when (file-exists-p (concat nnsoup-directory file))
+	(when (file-exists-p (expand-file-name file nnsoup-directory))
 	  (save-excursion		; Load the file.
 	    (set-buffer (get-buffer-create buffer-name))
 	    (buffer-disable-undo)
 	    (push (cons nnsoup-current-group (current-buffer)) nnsoup-buffers)
-	    (nnheader-insert-file-contents (concat nnsoup-directory file))
+	    (nnheader-insert-file-contents
+	     (expand-file-name file nnsoup-directory))
 	    (current-buffer))))))
 
 (defun nnsoup-file (prefix &optional message)
   (expand-file-name
-   (concat nnsoup-directory prefix (if message ".MSG" ".IDX"))))
+   (concat prefix (if message ".MSG" ".IDX"))
+   nnsoup-directory))
 
 (defun nnsoup-message-buffer (prefix)
   (nnsoup-index-buffer prefix 'msg))
@@ -644,25 +651,25 @@ backend for the messages.")
 (defun nnsoup-article-to-area (article group)
   "Return the area that ARTICLE in GROUP is located in."
   (let ((areas (cddr (assoc group nnsoup-group-alist))))
-    (while (and areas (< (cdaar areas) article))
+    (while (and areas (< (cdar (car areas)) article))
       (setq areas (cdr areas)))
     (and areas (car areas))))
 
 (defvar nnsoup-old-functions
-  (list message-send-mail-function message-send-news-function))
+  (list message-send-mail-real-function message-send-news-function))
 
 ;;;###autoload
 (defun nnsoup-set-variables ()
   "Use the SOUP methods for posting news and mailing mail."
   (interactive)
   (setq message-send-news-function 'nnsoup-request-post)
-  (setq message-send-mail-function 'nnsoup-request-mail))
+  (setq message-send-mail-real-function 'nnsoup-request-mail))
 
 ;;;###autoload
 (defun nnsoup-revert-variables ()
   "Revert posting and mailing methods to the standard Emacs methods."
   (interactive)
-  (setq message-send-mail-function (car nnsoup-old-functions))
+  (setq message-send-mail-real-function (car nnsoup-old-functions))
   (setq message-send-news-function (cadr nnsoup-old-functions)))
 
 (defun nnsoup-store-reply (kind)
@@ -722,7 +729,7 @@ backend for the messages.")
   (unless nnsoup-replies-list
     (setq nnsoup-replies-list
 	  (gnus-soup-parse-replies
-	   (concat nnsoup-replies-directory "REPLIES"))))
+	   (expand-file-name "REPLIES" nnsoup-replies-directory))))
   (let ((replies nnsoup-replies-list))
     (while (and replies
 		(not (string= kind (gnus-soup-reply-kind (car replies)))))
@@ -801,7 +808,8 @@ backend for the messages.")
     ;; Sort and delete the files.
     (setq non-files (sort non-files 'string<))
     (map-y-or-n-p "Delete file %s? "
-		  (lambda (file) (delete-file (concat nnsoup-directory file)))
+		  (lambda (file) (delete-file
+				  (expand-file-name file nnsoup-directory)))
 		  non-files)))
 
 (provide 'nnsoup)
